<?php
/*


// Usage
$k = new kAPI('APIKey', 'secretKey');
try {
    $params = array('foo' => 'bar');
    $result = $k->call('method', $params);
}
// Can not do try/catch to let default handler kick in
catch (kAPIException $e) {
    echo $e->getMessage();
    echo 'Server response:';
    var_dump($e->serverResponse);
}



*/
/**
 * kAPIResponseCodes
 * Set of constants for response codes and messages
 * 
 * @author 
 */
final class kAPIResponseCodes {
    
    const SUCCESS               =  1;
    const NOT_AUTHORISED        =  2;
    const NO_SUCH_METHOD        =  4;
    const NO_SUCH_FORMAT        =  8;
    const ACCOUNT_SUSPENDED     = 16;
    const INVALID_REQUEST       = 32;
    const ERROR                 = 64;
    
    private static $CODE_MESSAGES = array(
        1 => 'Success',
        2 => 'Not authorised',
        4 => 'No such method',
        8 => 'No such format',
        16 => 'Account suspended',
        32 => 'Invalid request',
        64 => 'Unknown server error'
    );
    
    private function __construct() {}
    private function __clone() {}
    
    public static function messageForCode($responseCode) {
        return (isset(self::$CODE_MESSAGES[$responseCode]))
            ? self::$CODE_MESSAGES[$responseCode]
            : 'Unknown';
    }
    
}

class kAPIException extends Exception {
    
    public $serverResponse;

    public function __construct($message, $response) {
        parent::__construct($message, $response->responseCode);
        $this->serverResponse = $response;
    }

}

class kAPI {
    
    const URL = 'https://api.kumulos.com';
    
    private 
        $APIKey,
        $secretKey,
        $salt,
        $hashedKey,
        $throwExceptionsOnError;
    
    public function __construct($APIKey, $secretKey, $throwExceptionsOnError = true) {
        $this->APIKey = $APIKey;
        $this->secretKey = $secretKey;
        $this->salt = rand(1,999999);
        $this->hashedKey = md5($this->secretKey . $this->salt);
        $this->throwExceptionsOnError = $throwExceptionsOnError;
    }
    
    public function call($methodName, $params = array()) {
        $url = kAPI::URL . '/' . $this->APIKey . '/' . $methodName . '.json';
        
        $params = array(
            'hashedKey' => $this->hashedKey,
            'salt'      => $this->salt,
            'params'    => $params
        );
        
        $params = http_build_query($params);
        
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION  ,1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER  ,1);
        $response = curl_exec($ch);
        curl_close($ch);
        
        $response = json_decode($response);
        if ($response->responseCode != kAPIResponseCodes::SUCCESS) {
            if ($this->throwExceptionsOnError) {
                throw new kAPIException($response->responseMessage, $response);
            }
            else {
                trigger_error($response->responseMessage);
            }
        }
        else {
            return $response->payload;
        }
    }
}